
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "rand.h"
#include "screen.h"
#include "task_arm.h"
#include "main.h"

//#include "header\font\BGFont.h"


//Mode3
//bg wi
//Spr 

//---------------------------------------------------------------------------
u8 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:    StateInit();    break;
			case STATE_RESET:   StateReset();   break;
			case STATE_FADEIN:  StateFadeIn();  break;
			case STATE_FADEOUT: StateFadeOut(); break;
			case STATE_MUSIC:   StateMusic();   break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);
	InitGBFS();

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	REG_TM2D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC half;		//p

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(NULL, &half);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitGBFS()
{
	if( GBFSInit() == FALSE )
	{
		SystemError("[Err] GBFS Not Found.", NULL);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	_Srand(0x12345678);

	ScreenImgInit();
	ScreenImgLoad();

	ScreenImgFadeIn(BLEND_MODE_DARK, 0, 15, 0);
	ScreenImgUpdate();

	State = STATE_FADEOUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateFadeIn()
{
	ScreenImgFadeIn(BLEND_MODE_DARK, 0, 15, 20);

	State = STATE_FADEOUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateFadeOut()
{
	ScreenImgFadeOut(20);

	State = STATE_MUSIC;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateMusic()
{
	u8* pData = GBFSGetFilePointer("miku.8ad");
	u32 size  = GBFSGetFileSize();

	if(pData == NULL)
	{
		SystemError("[Err] Music File Not Found.", NULL);
	}

	AdInit();
	AdStart(pData, size, FALSE);

	TaskInit();
	for(;;)
	{
		WaitForVsync();

		TaskInput();
		TaskExec();

		if( AdIsEndData() == TRUE && TaskIsSprAllFree() == TRUE )
		{
			break;
		}
	}

	State = STATE_FADEOUT;
}




//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{
	LibMode3SetScreenBuffer((u16*)VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0,  0, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(0, 11, (u8*)msg2, COLOR_WHITE);

	for(;;){}
}
