
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "rand.h"
#include "screen.h"
#include "main.h"

#include "task_arm.h"

//---------------------------------------------------------------------------
ST_TASK Task;

//---------------------------------------------------------------------------
IWRAM_CODE void TaskInit()
{
	_Memset((u8*)&Task, 0x00, sizeof(ST_TASK));
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskExec()
{
	ST_TASK* p = &Task;
	u16 i;

	//TODO Xg\ɂ2[vKvȂǁAʓ|Ȃ̂łĂȂłiOOG

	for(i=0; i<SPR_MAX_CNT; i++)
	{
		if(p->spr[i].isReg == TRUE)
		{
			p->spr[i].pAct(&p->spr[i]);
		}
	}

	for(i=0; i<SPR_MAX_CNT; i++)
	{
		if(p->spr[i].isReg == TRUE)
		{
			p->spr[i].pMov(&p->spr[i]);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskInput()
{
	u16 key = KeyGet();

	if(key == KEY_NONE)
	{
		return;
	}
	if(Task.regCnt >= TASK_MAX_REG_CNT)
	{
		return;
	}

	u16 y     = _UMod(_Rand(), SCREEN_CY - SPR_CY);
	u16 sprNo = TaskGetFreeSpr();

	if(key & KEY_A)
	{
		TaskRegSpr(sprNo, SPR_MIKKU_STR1_TILENO, y, &TaskExecSprAct1, &TaskExecSprMov1);
		Task.regCnt++;
	}
	if(key & KEY_B)
	{
		TaskRegSpr(sprNo, SPR_MIKKU_STR2_TILENO, y, &TaskExecSprAct1, &TaskExecSprMov1);
		Task.regCnt++;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE bool TaskIsFreeSpr()
{
	ST_TASK* p = &Task;
	u16 i;

	for(i=0; i<SPR_MAX_CNT; i++)
	{
		if(p->spr[i].isReg == FALSE)
		{
			return TRUE;
		}
	}

	return FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 TaskGetFreeSpr()
{
	ST_TASK* p = &Task;
	u16 i;

	for(i=0; i<SPR_MAX_CNT; i++)
	{
		if(p->spr[i].isReg == FALSE)
		{
			return i;
		}
	}

	SystemError("[Err] TaskGetFreSpr()", NULL);
	return 0;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskRegSpr(u16 sprNo, u16 tileNo, s16 y, void* pAct, void* pMov)
{
	ST_SPR* p = &Task.spr[sprNo];

	p->isReg  = TRUE;
	p->x      = SCREEN_CX;
	p->y      = y;
	p->cx     = SPR_CX;
	p->cy     = SPR_CY;
	p->xMove  = _UMod(_Rand(), 5) + 2;
	p->sprNo  = sprNo;
	p->tileNo = tileNo;
	p->isNext = FALSE;
	p->pAct   = pAct;
	p->pMov   = pMov;

	SpriteSetSize(p->sprNo, SP_SIZE_64, SP_WIDE, SP_COLOR_16);
	SpriteSetChr (p->sprNo, p->tileNo);
	SpriteMove   (p->sprNo, p->x, p->y);

	Task.sprCnt++;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskRegSpr2(u16 sprNo, u16 tileNo, s16 x, s16 y, void* pAct, void* pMov, s16 xMove)
{
	ST_SPR* p = &Task.spr[sprNo];

	TaskRegSpr(sprNo, tileNo, y, pAct, pMov);
	p->x     = x;
	p->xMove = xMove;
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskFreeSpr(ST_SPR* p)
{
	p->isReg = FALSE;
	Task.sprCnt--;
}




//---------------------------------------------------------------------------
IWRAM_CODE void TaskExecSprAct1(ST_SPR* p)
{
	if(p->x < (0 - p->cx))
	{
		TaskFreeSpr(p);
	}
	else if(p->isNext == FALSE && p->x <= (SCREEN_CX - p->cx))
	{
		p->isNext = TRUE;

		u16 sprNo = TaskGetFreeSpr();
		TaskRegSpr2(sprNo, p->tileNo+8, p->x + p->cx, p->y, TaskExecSprAct2, TaskExecSprMov1, p->xMove);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskExecSprAct2(ST_SPR* p)
{
	if(p->x < (0 - p->cx))
	{
		TaskFreeSpr(p);
	}
	else if(p->isNext == FALSE && p->x <= (SCREEN_CX - p->cx))
	{
		p->isNext = TRUE;

		u16 sprNo = TaskGetFreeSpr();
		TaskRegSpr2(sprNo, p->tileNo+8, p->x + p->cx, p->y, TaskExecSprAct3, TaskExecSprMov1, p->xMove);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskExecSprAct3(ST_SPR* p)
{
	if(p->x < (0 - p->cx))
	{
		TaskFreeSpr(p);
		Task.regCnt--;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void TaskExecSprMov1(ST_SPR* p)
{
	p->x -= p->xMove;
	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
IWRAM_CODE bool TaskIsSprAllFree()
{
	return Task.regCnt == 0 ? TRUE : FALSE;
}
